// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/Scene.hpp"
#include "scene/components/PlayerController.hpp"
#include "scene/components/GameCamera.hpp"
#include "scene/components/GameCamera.hpp"
#include "scene/components/display/MeshRenderer.hpp"
#include "scene/components/display/MeshHost.hpp"
#include "scene/components/display/material/SimpleTexturedMaterial.hpp"
#include "scene/components/example/ExampleSpin.hpp"
#include "scene/components/physics/3d/SphereCollider.hpp"
#include "scene/components/physics/3d/CubeCollider.hpp"
#include "scene/components/physics/3d/CapsuleCollider.hpp"
#include "display/mesh/SphereMesh.hpp"
#include "display/mesh/CapsuleMesh.hpp"

namespace Dawn {
  class HelloWorldScene : public Scene {
    protected:
      Camera *camera;

      void stage() override {

        auto playerItem = this->createSceneItem();
        auto player = playerItem->addComponent<PlayerController>();
        playerItem->addComponent<MeshRenderer>();
        auto meshHost = playerItem->addComponent<MeshHost>();
        playerItem->addComponent<SimpleTexturedMaterial>();
        auto hitbox = playerItem->addComponent<CapsuleCollider>();
        playerItem->addComponent<CharacterController3D>();
        CapsuleMesh::create(&meshHost->mesh, hitbox->radius, hitbox->height);

        playerItem->transform.setLocalPosition(glm::vec3(0, 4, 0));

        auto wall = this->createSceneItem();
        auto wallHitbox = wall->addComponent<CubeCollider>();
        // wall->transform.setLocalPosition(glm::vec3(-5, -1, -5));
        wallHitbox->min = -(wallHitbox->max = glm::vec3(5, 0.25f, 5));

        camera = Camera::create(this);
        camera->transform->lookAt(glm::vec3(10, 10, 10), glm::vec3(0, 0, 0));
        // auto gameCamera = camera->item->addComponent<GameCamera>();
        // gameCamera->player = player;
      }
      
      std::vector<Asset*> getRequiredAssets() override {
        auto assMan = &this->game->assetManager;
        std::vector<Asset*> assets;
        return assets;
      }

    public:
      HelloWorldScene(DawnGame *game) : Scene(game) {}
  };
}